@@ -11,10 +11,10 @@ use strict;
 use warnings;
 
 package Net::Airbrake::V2;
-# git description: v0.01-2-g1348790
+# git description: v0.02-4-g4971881
 
 our $AUTHORITY = 'cpan:RWSTAUNER';
-$Net::Airbrake::V2::VERSION = '0.02';
+$Net::Airbrake::V2::VERSION = '0.03';
 # ABSTRACT: Airbrake Notifier API V2 Client
 
 use Net::Airbrake 0.02;
@@ -51,18 +51,45 @@ sub _make_vars {
 
   return ($parent => [
     {
-      var => [
-        map {
-          {
-            key     => $_,
-            content => $vars->{$_},
-          }
-        } keys %{ $vars }
-      ]
+      $self->_var_content($vars),
     }
   ]);
 }
 
+sub _var_content {
+  my ($self, $v) = @_;
+
+  return (content => ref($v) ? $self->stringify_ref($v) : $v)
+    unless ref($v) eq 'HASH';
+
+  return (
+    var => [
+      map {
+        {
+          key     => $_,
+          $self->_var_content($v->{$_}),
+        }
+      } keys %$v
+    ]
+  );
+}
+
+
+our $Dumper;
+sub stringify_ref {
+  my ($self, $value) = @_;
+  ($Dumper ||= do {
+    require Data::Dumper;
+    Data::Dumper->new([])
+      ->Indent(0)
+      ->Useqq(1)
+      ->Terse(1)
+      ->Sortkeys(1)
+  })
+    ->Values([$value])
+    ->Dump;
+}
+
 
 sub convert_request {
   my ($self, $req, $opts) = @_;
@@ -139,6 +166,10 @@ sub _xml_encode {
     XMLDecl  => q[<?xml version="1.0" encoding="utf-8"?>],
   );
 
+  # decode_json should return a structure with character strings,
+  # and XML::Simple doesn't encode until writing to a file handle.
+  utf8::encode($xml);
+
   return $xml;
 }
 
@@ -226,7 +257,7 @@ Net::Airbrake::V2 - Airbrake Notifier API V2 Client
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -254,6 +285,15 @@ See L<Net::Airbrake> for descriptions of methods and arguments.
 
 =head1 METHODS
 
+=head2 stringify_ref
+
+The values of the the "var" hashes ("params", "session", and "environment")
+that are references (other than hashes) will stringified.
+
+This is currently done with C<Data::Dumper>
+which is similar to the way the ruby gem dumps structures.
+The format is subject to change.
+
 =head2 convert_request
 
   $client->convert_request(\%v3_request);
